// (C) 2025 GoodData Corporation
import React from "react";
import isEqual from "react-fast-compare";
function createStore(initialState) {
    let state = initialState;
    const subscribers = new Set();
    const getState = () => state;
    const setState = (setter) => {
        const prevState = state;
        state = setter(prevState);
        subscribers.forEach((subscriber) => subscriber(state, prevState));
    };
    const subscribe = (subscriber) => {
        subscribers.add(subscriber);
        return () => {
            subscribers.delete(subscriber);
        };
    };
    return {
        getState,
        setState,
        subscribe,
    };
}
/**
 * Creates a context store.
 * This allows components inside the context to select slices of the store and only rerender when the slice changes.
 *
 * Provides a Provider and a slice selector hook.
 *
 * @internal
 */
export const createContextStore = (name) => {
    const Context = React.createContext(null);
    const Provider = ({ value, children }) => {
        const [store] = React.useState(() => createStore(value));
        React.useEffect(() => {
            store.setState(() => value);
        }, [store, value]);
        return React.createElement(Context.Provider, { value: store }, children);
    };
    const useContextStoreOptional = (selector = (state) => state, equalityFn = isEqual) => {
        const store = React.useContext(Context);
        const [selectedState, setSelectedState] = React.useState(() => store ? selector(store.getState()) : undefined);
        React.useEffect(() => {
            if (!store) {
                return undefined;
            }
            return store.subscribe((newState, prevState) => {
                const newSelectedState = selector(newState);
                const prevSelectedState = selector(prevState);
                if (!equalityFn(newSelectedState, prevSelectedState)) {
                    setSelectedState(newSelectedState);
                }
            });
        }, [selector, equalityFn, store]);
        return selectedState;
    };
    const useContextStore = (selector = (state) => state, equalityFn = isEqual) => {
        const store = React.useContext(Context);
        if (store === null) {
            throw new Error(`Context store '${name}' must be used within a Provider`);
        }
        return useContextStoreOptional(selector, equalityFn);
    };
    const createSelector = (selector) => selector;
    return Object.assign(Provider, {
        useContextStore,
        useContextStoreOptional,
        createSelector,
    });
};
//# sourceMappingURL=contextStore.js.map